# Generated by Django 2.2.3 on 2019-07-09 15:34

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='ExtentedSettings',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(help_text='Configuration name', max_length=1024, verbose_name='Configuration')),
                ('key', models.CharField(max_length=255, verbose_name='Parameter name')),
                ('value', models.TextField(blank=True, null=True, verbose_name='Value')),
                ('enabled', models.BooleanField(default=True, verbose_name='Activation')),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='Created at')),
                ('updated_at', models.DateTimeField(auto_now=True, verbose_name='Updated_at')),
            ],
            options={
                'verbose_name': 'Parameter',
                'verbose_name_plural': 'Parameters',
                'db_table': 'django_extended_settings',
                'ordering': ('name', 'key'),
            },
        ),
    ]



from django.db import models
try:
    from django.utils.translation import ugettext_lazy as _
except:
    from django.utils.translation import gettext_lazy as _

# ----------------------------------------------------------------------------------------------------------------------
# - Models for database settings
# ----------------------------------------------------------------------------------------------------------------------
class ExtentedSettings(models.Model):
    """
    All database application parameters
    """
    class Meta:
        db_table = "django_extended_settings"
        verbose_name = _('Parameter')
        verbose_name_plural = _(u'Parameters')
        ordering = ("name", "key", )

    # fields type
    TYPE_BOOLEAN = "boolean"
    TYPE_NUMBER = "number"
    TYPE_VARCHAR = "varchar"
    TYPE_TEXT = "text"
    TYPE_CHOICES = (
        (TYPE_BOOLEAN, _("Yes/No")),
        (TYPE_NUMBER, _("Number")),
        (TYPE_VARCHAR, _("Text field")),
        (TYPE_TEXT, _("Multiline text field")),
    )

    # primary fields
    name = models.CharField(max_length=1024, verbose_name=_("Configuration"), help_text=_("Configuration name"))
    key = models.CharField(max_length=255, verbose_name=_("Parameter name"))
    value = models.TextField(blank=True, null=True, verbose_name=_("Value"))
    # type = models.CharField(max_length=50,
    #                         choices=TYPE_CHOICES,
    #                         default=TYPE_VARCHAR,
    #                         help_text=_("Field type"),
    #                         verbose_name=_("Type"))

    # model management
    enabled = models.BooleanField(default=True, verbose_name=_("Activation"))
    created_at = models.DateTimeField(auto_now_add=True, verbose_name=_("Created at"))
    updated_at = models.DateTimeField(auto_now=True, verbose_name=_("Updated_at"))

    def __str__(self):
        return self.name

    @staticmethod
    def get(key, default=""):
        """
        get configuration value

        :param key: configuration key
        :param default: default value if error
        :return: String
        """
        try:
            o = ExtentedSettings.objects.get(key=key)
            return str(o.value)

        except:
            # not found!
            return default

    @staticmethod
    def get_boolean(key, default=False):
        """
        get configuration value (boolean)

        :param key: configuration key
        :param default: default value if error
        :return: String
        """
        try:
            value = ExtentedSettings.get(key, "no")
            if value.lower().strip() in ("yes", "oui", "true", "1", "si", "ja", "y", "o", "t", "s", "j"):
                return True
            return False

        except:
            # pas trouvé!
            return default

    @staticmethod
    def get_int(key, default=0):
        """
        get configuration value (int)

        :param key: configuration key
        :param default: default value if error
        :return: String
        """
        try:
            return int(ExtentedSettings.get(key, default))

        except:
            # pas trouvé!
            return default

    @staticmethod
    def get_float(key, default=0.0):
        """
        get configuration value (float)

        :param key: configuration key
        :param default: default value if error
        :return: String
        """
        try:
            value = ExtentedSettings.get(key, default).replace(",", ".")
            return float(value)

        except:
            # pas trouvé!
            return default

    # ------------------------------------------------------------------------------------------------------------------
    def custom_field_value(self, cut=50):
        """
        Display parameter value strip
        :param cut: strip number
        :return: str
        """
        value = str(self.value).replace('\n', ' ').replace('\r', '')
        return value if len(value) < cut else value[0:cut] + "..."
    custom_field_value.short_description = _("Value")



# --------------------------------------------------------------------------------------------
# - Django Extended Settings
# -
# - MickBad
# --------------------------------------------------------------------------------------------

import os
import sys
import imp
import glob
from django.apps import AppConfig
from django.conf import settings


# --------------------------------------------------------------------------------------------
# - Import extended settings apps
# --------------------------------------------------------------------------------------------
class ExtendedSettingsConfig(AppConfig):
    name = "extended_settings"
    verbose_name = "Extended Settings"

    def __init__(self, app_name, app_module):
        """
        Function to initialize other configuration settings for django project
        Other configurations must be set in /path/to/project/djangoproject/settings.d/*_settings.py

        Project
        |_ apps1
        |_ ...
        |_ project
           |_ settings.py
           |_ urls.py
           |_ ...
           |_ settings.d
              |_ one_settings.py
              |_ second_settings.py

        :return: None
        """
        # init AppConfig
        super(ExtendedSettingsConfig, self).__init__(app_name, app_module)

        # get project root path
        try:
            BASE_DIR = settings.BASE_DIR

        except AttributeError:
            raise Exception("""
** Django Extends Settings **
Please upgrade to Django 1.8+ or create on your project settings.py the following constant:

# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
            """)

        # get project information
        django_projectname = settings.SETTINGS_MODULE
        django_projectdir = os.path.join(BASE_DIR, django_projectname.split(".")[0])

        # read only valid settings extends files (sort files by alpha)
        list_settings_file = glob.glob(os.path.join(django_projectdir, "settings.d/*_settings.py"))
        list_settings_file.sort()
        for file_settings in list_settings_file:
            # log
            print("* Reading extends settings from {}".format(os.path.basename(file_settings)))

            # module transformation
            module_name = file_settings.split(".py")[0]
            module = imp.new_module(module_name)
            module.__file__ = file_settings
            sys.modules[module_name] = module

            # get settings configuration for locals in sub configuration
            local_settings = {}
            instance_settings = settings.__dict__['_wrapped'].__dict__
            for k in instance_settings:
                local_settings[k] = instance_settings[k]

            # read and exec settings with current settings context
            exec(open(file_settings, "rb").read(), globals(), local_settings)

            # transform extends settings to global settings
            for var in local_settings:
                # TODO: please fix when settings.d import lib (like from mblibs.fast import FastSettings)
                try:
                    val = local_settings[var]
                    if type(val) is str:
                        exec("settings.{var}='{val}'".format(var=var, val=val.replace("'", "\\'")))
                    else:
                        exec("settings.{var}={val}".format(var=var, val=val))

                except:
                    pass


    def ready(self):
        pass




from django.contrib import admin
from extended_settings.models import ExtentedSettings

# ----------------------------------------------------------------------------------------------------------------------
# - Création de l'administration des paramètres
# ----------------------------------------------------------------------------------------------------------------------
@admin.register(ExtentedSettings)
class ExtentedSettingsAdmin(admin.ModelAdmin):
    """
    Parameters Administration
    """
    readonly_fields = ("name", "key", "updated_at")
    list_display = ("name", "custom_field_value", "updated_at")

    fieldsets = (
        (None, {
            "fields": ("name",
                       "key",
                       "value",
                       "updated_at",)
        }),
    )

    def has_add_permission(self, request, obj=None):
        return False

    def has_delete_permission(self, request, obj=None):
        return False




#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from setuptools import setup, find_packages

import extended_settings

# description
long_description = open('README.md').read()

# setup
setup(
    name='django-extended-settings',
    version=extended_settings.__version__,

    packages=find_packages(exclude=('tests.*', 'tests', 'example')),

    author="MickBad",
    author_email="prog@mickbad.com",
    description="Extended Settings for Django Project with settings.d/",

    long_description=long_description,
    long_description_content_type='text/markdown',

    install_requires=[
        "Django>=2.0",
    ],

    python_requires=">=3.5",

    # activate MANIFEST.in
    include_package_data=True,

    # github source
    url='https://github.com/mickbad/django-extended-settings',
    download_url='https://pypi.org/project/django-extended-settings/',

    # https://pypi.python.org/pypi?%3Aaction=list_classifiers.
    classifiers=[
        "Programming Language :: Python",
        "Development Status :: 4 - Beta",
        "License :: OSI Approved :: BSD License",
        "Natural Language :: French",
        "Operating System :: OS Independent",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: Software Development :: Libraries",
        "Topic :: Utilities",
    ],

    license="BSD",

    keywords="django development tools settings",
)
